%----------------------------------------------------------------------
% Script generates a scatter plot of human sketch recognition accuracy as
% shown in Figure 4 of Eitz et al. -- How do humans sketch objects?
%
% (c) Mathias Eitz 2012
% Contact: m.eitz@tu-berlin.de
%----------------------------------------------------------------------

clear;

filename = 'human_recognition.mat';

load(filename, 'humanRecognition');
D = humanRecognition(2:end,:);


[G GN] = grp2idx(D(:,16));
G = repmat(G,4,1);

numHits = size(G,1);
numWorkers = numel(GN);


groundtruth = reshape(D(:,[29,31,33,35]),numHits, 1);
pred = reshape(D(:,[38,41,44,47]),numHits,1);

comments = D(:,48);
comments = comments(~(strcmp(comments,'')));

% a 1 in theR vector means that the corresponding
% groundtruth entry has been correctly recognized by the humans
R = strcmp(pred, groundtruth);


% ---------------------------------
% global accuracy over all workers
acc_global = sum(R(:)) / numel(R);
disp(['overall global accuracy=' num2str(acc_global)]);


% ------------------------------------------------------
% Accuracy per worker

% Create the matrix W, such that W(i,j) == 1 means that
% worker with unique id j has done HIT with index i.
N = repmat(G,1,numWorkers);
W = (N == repmat(1:numWorkers, numHits,1));

% H is a row vector of size numHits with H(i) = number of HITs done
% done for worker with id i
H = sum(W,1);


% W(:,i) defines which HITs worker i has done
% So the and operation gives us a matrix C where
% C(:,i) indicates the HITS that worker i has correctly
% answered.
RR = repmat(R,1,numWorkers);
C = and(RR,W);
accPerWorker = sum(C,1) ./ H;


% Remove all workers that have done less than k HITS
globals = zeros(0);
totalNumWorkers = zeros(0);
range = min(H):4:max(H);
for i=range
    ix = H >= i;
    ACW2 = sum(C(:,ix),1) ./ H(ix);
    globals(end+1) = sum(sum(C(:,ix))) / sum(H(ix));
    totalNumWorkers(end+1) = sum(ix);
end


% ----------------------------------------------------------------------------------------------
% generate a final overall number, by removing those workers that have
% worked on less than minNumHits sketches. The reasoning behind this is
% that people need a couple of runs to learn the hierarchy
minNumSketches = 40;
acc = globals(minNumSketches/4);
disp(['avg accuracy for workers that have done at least ' num2str(minNumSketches) ' sketches: ' num2str(acc)]);



% -------------------------------------------------------
% Plot accuracy when gradually removing workers that have
% done less than k HITS
figure

green = [77 141 64]/255;
blue = [0,142,246]/255;

set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperSize', [3.33 2]);
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperPosition', [0 0 3.33 2]);  %left, bottom, width, height
set(gca, 'FontSize', 8);

grid on
hold on
ylim([0.2 1]);
title('human sketch recognition accuracy');
ylabel('accuracy');

[v ix] = sort(accPerWorker);

scatter(H(ix),v,5,green,'filled');


% don't plot the last two points such that the last point
% corresponding to the worker with the most sketches is clearly
% visible
scatter(range(1:end-2),globals(1:end-2),1,blue);

title('accuracy vs. #sketches');
xlabel('#sketches');
ylabel('accuracy');

legend('unique worker', 'acc of workers with >= #sketches', 'Location', 'SouthEast');
print(gcf, '-dpdf', '-r300', 'recognition_accuracy.pdf');